# snappy-proposed-selftest-agent
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import kombu
from testtools import TestCase

from snappy_proposed_selftest_agent import (constants, read_config)
from snappy_proposed_selftest_agent.worker import CoreSelftestAgentWorker


class WorkerTests(TestCase):
    def test_can_read_message(self):
        conn = kombu.Connection('memory:///')
        queue_message(conn, {'test': 'value'})

        build_worker = LoggingConsumer()
        result_worker = LoggingConsumer()
        config = read_config()
        q = CoreSelftestAgentWorker(conn, config,
                                    build_worker, result_worker)
        next(q.consume(limit=1, timeout=5.0))
        self.assertEqual(build_worker.messages_seen[0].get('test'), 'value')
        self.assertEqual(result_worker.messages_seen[0].get('test'), 'value')


class LoggingConsumer(object):
    """A consumer callback object that acks and logs all received payloads."""
    def __init__(self):
        self.messages_seen = []

    def __call__(self, message):
        self.messages_seen.append(message)


def queue_message(conn, message):
    exchange = kombu.Exchange(constants.INPUT_EXCHANGE,
                              type='fanout', durable=False)
    queue = kombu.Queue(constants.INPUT_QUEUE, exchange)
    with conn.SimpleQueue(queue) as q:
        q.put(message)
