# snappy-proposed-selftest-agent
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import kombu
import urllib

from kombu.mixins import ConsumerMixin

from snappy_proposed_selftest_agent import constants


class CoreSelftestAgentWorker(ConsumerMixin):

    def __init__(self, connection, config, build_worker, result_worker):
        self.config = config
        self.connection = connection
        self.build_worker = build_worker
        self.result_worker = result_worker

        exchange = kombu.Exchange(constants.INPUT_EXCHANGE,
                                  type='fanout', durable=False)
        self.queue = kombu.Queue(constants.INPUT_QUEUE, exchange)

    def get_consumers(self, Consumer, channel):
        """Return consumers instances for all configured queues."""
        queues = [self.queue]
        return [kombu.Consumer(
            channel, queues=queues, callbacks=[self.process])]

    def process(self, body, message):
        """ Process incoming messages about candidate packages

        TODO: Check if we care about this package, if so, send a message
        to the image builder to create the image and start the tests, and
        a message to the results exchange to mark it as in progress.

        For now, just blindly post messages for these
        """

        request_id = kombu.uuid()

        # Build the message body to pass to the image builder
        image_build_body = body.copy()
        image_build_body['test_branch'] = constants.TEST_BRANCH
        image_build_body['channel'] = constants.CHANNEL
        image_build_body['device'] = constants.DEVICE
        image_build_body['test_name'] = constants.TEST_NAME
        self.build_worker(image_build_body)

        # Build the message body to pass to the results exchange
        results_body = body.copy()
        swift_url = self.config.get('swift', 'public_url')
        results_path = 'snappy-selftest-{}/results.tgz'.format(request_id)
        request_url = urllib.parse.urljoin(swift_url, results_path)
        results_body['request_url'] = request_url
        results_body['status'] = 'RUNNING'
        results_body['test_name'] = constants.TEST_NAME
        self.result_worker(results_body)
